--- asterisk/channels/chan_modem.c.~1.6.~	2003-09-08 18:48:06.000000000 +0200
+++ asterisk/channels/chan_modem.c	2003-10-29 16:51:02.000000000 +0100
@@ -63,6 +63,12 @@
 /* Default Listen */
 static char incomingmsn[AST_MAX_EXTENSION]="";
 
+/* Default CID_PREFIX */
+static char cid_prefix[5]="";
+
+/* Default skip dtmf */
+static int allowoutbanddtmf = 0;
+
 /* Default group */
 static unsigned int cur_group = 0;
 
@@ -707,6 +713,8 @@
 		strncpy(tmp->language, language, sizeof(tmp->language)-1);
 		strncpy(tmp->msn, msn, sizeof(tmp->msn)-1);
 		strncpy(tmp->incomingmsn, incomingmsn, sizeof(tmp->incomingmsn)-1);
+		strncpy(tmp->cid_prefix, cid_prefix, sizeof(tmp->cid_prefix)-1);
+		tmp->allowoutbanddtmf = allowoutbanddtmf;
 		strncpy(tmp->dev, iface, sizeof(tmp->dev)-1);
 		/* Maybe in the future we want to allow variable
 		   serial settings */
@@ -909,6 +917,10 @@
 			strncpy(msn, v->value, sizeof(msn)-1);
 		} else if (!strcasecmp(v->name, "incomingmsn")) {
 			strncpy(incomingmsn, v->value, sizeof(incomingmsn)-1);
+		} else if (!strcasecmp(v->name, "cidprefix")) {
+			strncpy(cid_prefix, v->value, sizeof(cid_prefix)-1);
+		} else if (!strcasecmp(v->name, "allowoutbanddtmf")) {
+                        allowoutbanddtmf = atoi(v->value);
 		} else if (!strcasecmp(v->name, "language")) {
 			strncpy(language, v->value, sizeof(language)-1);
 		} else if (!strcasecmp(v->name, "group")) {
--- asterisk/channels/chan_modem_i4l.c.~1.3.~	2003-08-13 17:25:16.000000000 +0200
+++ asterisk/channels/chan_modem_i4l.c	2003-11-10 15:11:34.000000000 +0100
@@ -42,6 +42,24 @@
 	NULL
 };
 
+/* for in-band dtmf */
+short * dtmf_buf[16];
+const int f_low[16] = { 941, 697, 697, 697, 770, 770, 770, 852,
+			852, 852, 941, 941, 697, 770, 852, 941 };
+const int f_high[16] = { 1336, 1209, 1336, 1477, 1209, 1336,
+			 1477, 1209, 1336, 1477, 1209, 1477,
+			 1633, 1633, 1633, 1633 };
+const float time_on = 0.2;
+const float time_off = 0.1;
+const int sample_rate = 8000;
+const int max_value = 8000;
+const float my_pi = 3.14159265;
+//int num_samples = (int)(sample_rate*(time_on+time_off));
+int num_samples = 2400;
+int dtmf_tosend=0;        /* set to number of samples left of digit dtmf_digit_tosend to send */
+int dtmf_digit_tosend=0; /* the digit to send (index of dtmf_buf) */
+
+
 static int i4l_setdev(struct ast_modem_pvt *p, int dev)
 {
 	char cmd[80];
@@ -132,6 +150,7 @@
 
 static int i4l_init(struct ast_modem_pvt *p)
 {
+        int i,t;
 	char cmd[256];
 	if (option_debug)
 		ast_log(LOG_DEBUG, "i4l_init()\n");
@@ -205,6 +224,19 @@
 		return -1;
 	}
 	p->escape = 0;
+
+	/* prepare for in-band dtmf */
+
+	// calculate the 16 DTMF tones   
+	for (i=0; i<16; i++) {
+	  dtmf_buf[i] = (short*)malloc(num_samples  * sizeof(short));
+	  for (t=0; t<sample_rate*time_on; t++)
+	    dtmf_buf[i][t] = (short)((sin(2*my_pi*f_low[i]*t/sample_rate) + sin(2*my_pi*f_high[i]*t/sample_rate)) * max_value);
+	  for (t=(int)(sample_rate*time_on); t<num_samples; t++)
+	    dtmf_buf[i][t] = 0;
+	}
+	
+
 	return 0;
 }
 
@@ -277,9 +309,13 @@
 	case '9':
 	case '*':
 	case '#':
+	  if(p->allowoutbanddtmf) {
 		ast_log(LOG_DEBUG, "DTMF: '%c' (%d)\n", esc, esc);
 		p->fr.frametype=AST_FRAME_DTMF;
 		p->fr.subclass=esc;
+	  } else {
+	    ast_log(LOG_DEBUG, "Skipped DTMF '%c'\n",esc);
+	  }
 		return &p->fr;
 	case 0: /* Pseudo signal */
 		return &p->fr;
@@ -326,7 +362,8 @@
 				return i4l_handle_escape(p, 'b');
 			} else
 			if (!strncasecmp(result, "CALLER NUMBER: ", 15 )) {
-				strncpy(p->cid, result + 15, sizeof(p->cid)-1);
+                                strncpy(p->cid, p->cid_prefix, sizeof(p->cid));
+				strncat(p->cid, result + 15, sizeof(p->cid)-1-strlen(p->cid_prefix));
 				return i4l_handle_escape(p, 0);
 			} else
 			if (!strcasecmp(result, "RINGING")) {
@@ -446,7 +483,12 @@
 		return -1;
 	}
 	for (x=0;x<f->datalen/2;x++) {
-		b = INT2ALAW(((short *)f->data)[x]);
+	        if(dtmf_tosend) {
+	          b = INT2ALAW(dtmf_buf[dtmf_digit_tosend][num_samples-dtmf_tosend]);
+	          dtmf_tosend--;
+	        } else {
+		  b = INT2ALAW(((short *)f->data)[x]);
+	        }
 		result[bpos++] = b;
 		if (b == CHAR_DLE)
 			result[bpos++]=b;
@@ -517,9 +559,83 @@
 {
 	char c[2];
 	if (p->ministate == STATE_VOICE) {
+#if 0
 		c[0] = CHAR_DLE;
 		c[1] = digit;
 		write(p->fd, c, 2);
+#else
+		dtmf_tosend = num_samples;
+		switch(digit) {
+		case '0':
+		  dtmf_digit_tosend=0;
+		  break;
+    
+		case '1':
+		  dtmf_digit_tosend=1;
+		  break;
+    
+		case '2':
+		  dtmf_digit_tosend=2;
+		  break;
+    
+		case '3':
+		  dtmf_digit_tosend=3;
+		  break;
+    
+		case '4':
+		  dtmf_digit_tosend=4;
+		  break;
+    
+		case '5':
+		  dtmf_digit_tosend=5;
+		  break;
+
+		case '6':
+		  dtmf_digit_tosend=6;
+		  break;
+    
+		case '7':
+		  dtmf_digit_tosend=7;
+		  break;
+    
+		case '8':
+		  dtmf_digit_tosend=8;
+		  break;
+    
+		case '9':
+		  dtmf_digit_tosend=9;
+		  break;
+    
+		case '*':
+		  dtmf_digit_tosend=10;
+		  break;
+    
+		case '#':
+		  dtmf_digit_tosend=11;
+		  break;
+    
+		case 'a':
+		case 'A':
+		  dtmf_digit_tosend=12;
+		  break;
+    
+		case 'b':
+		case 'B':
+		  dtmf_digit_tosend=13;
+		  break;
+    
+		case 'c':
+		case 'C':
+		  dtmf_digit_tosend=14;
+		  break;
+    
+		case 'd':
+		case 'D':
+		  dtmf_digit_tosend=15;
+		  break;
+		}
+#endif
+		ast_log(LOG_DEBUG, "Asked to send digit %c on %s\n", digit, p->dev);
 	} else
 		ast_log(LOG_DEBUG, "Asked to send digit but call not up on %s\n", p->dev);
 	return 0;
--- asterisk/include/asterisk/vmodem.h.~1.1.1.2.~	2003-03-10 07:00:17.000000000 +0100
+++ asterisk/include/asterisk/vmodem.h	2003-10-29 16:52:41.000000000 +0100
@@ -113,6 +113,10 @@
 	unsigned int group;
 	/*! Caller ID if available */
 	char cid[AST_MAX_EXTENSION];	
+        /*! Caller ID prefix */
+        char cid_prefix[5];
+        /*! Allow dtmf from network outband */
+        int allowoutbanddtmf;
 	/*! Dialed Number if available */
 	char dnid[AST_MAX_EXTENSION];	
 	/*! Modem initialization String */
